@@ -6,7 +6,7 @@
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
  *
- * http://www.apache.org/licenses/LICENSE-2.0
+ * https://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
@@ -126,6 +126,7 @@ static void avro_schema_free(avro_schema_t schema)
 		case AVRO_DOUBLE:
 		case AVRO_BOOLEAN:
 		case AVRO_NULL:
+		case AVRO_INVALID:
 			/* no memory allocated for primitives */
 			return;
 
@@ -136,7 +137,7 @@ static void avro_schema_free(avro_schema_t schema)
 				if (record->space) {
 					avro_str_free(record->space);
 				}
-				st_foreach(record->fields, HASH_FUNCTION_CAST record_free_foreach,
+				st_foreach(record->fields, (hash_function_foreach) record_free_foreach,
 					   0);
 				st_free_table(record->fields_byname);
 				st_free_table(record->fields);
@@ -151,7 +152,7 @@ static void avro_schema_free(avro_schema_t schema)
 				if (enump->space) {
 					avro_str_free(enump->space);
 				}
-				st_foreach(enump->symbols, HASH_FUNCTION_CAST enum_free_foreach,
+				st_foreach(enump->symbols, (hash_function_foreach) enum_free_foreach,
 					   0);
 				st_free_table(enump->symbols);
 				st_free_table(enump->symbols_byname);
@@ -188,7 +189,7 @@ static void avro_schema_free(avro_schema_t schema)
 		case AVRO_UNION:{
 				struct avro_union_schema_t *unionp;
 				unionp = avro_schema_to_union(schema);
-				st_foreach(unionp->branches, HASH_FUNCTION_CAST union_free_foreach,
+				st_foreach(unionp->branches, (hash_function_foreach) union_free_foreach,
 					   0);
 				st_free_table(unionp->branches);
 				st_free_table(unionp->branches_byname);
@@ -876,15 +877,7 @@ static int
 avro_schema_from_json_t(json_t *json, avro_schema_t *schema,
 			st_table *named_schemas, const char *parent_namespace)
 {
-#ifdef _WIN32
- #pragma message("#warning: Bug: '0' is not of type avro_type_t.")
-#else
- #warning "Bug: '0' is not of type avro_type_t."
-#endif
-  /* We should really have an "AVRO_INVALID" type in
-   * avro_type_t. Suppress warning below in which we set type to 0.
-   */
-	avro_type_t type = (avro_type_t) 0;
+	avro_type_t type = AVRO_INVALID;
 	unsigned int i;
 	avro_schema_t named_type = NULL;
 
@@ -1246,7 +1239,7 @@ avro_schema_from_json_root(json_t *root, avro_schema_t *schema)
 	/* json_dumpf(root, stderr, 0); */
 	rval = avro_schema_from_json_t(root, schema, named_schemas, NULL);
 	json_decref(root);
-	st_foreach(named_schemas, HASH_FUNCTION_CAST named_schema_free_foreach, 0);
+	st_foreach(named_schemas, (hash_function_foreach) named_schema_free_foreach, 0);
 	st_free_table(named_schemas);
 	return rval;
 }
@@ -1462,7 +1455,7 @@ avro_schema_t avro_schema_copy(avro_schema_t schema)
 	}
 
 	new_schema = avro_schema_copy_root(schema, named_schemas);
-	st_foreach(named_schemas, HASH_FUNCTION_CAST named_schema_free_foreach, 0);
+	st_foreach(named_schemas, (hash_function_foreach) named_schema_free_foreach, 0);
 	st_free_table(named_schemas);
 	return new_schema;
 }
@@ -1882,6 +1875,8 @@ avro_schema_to_json2(const avro_schema_t schema, avro_writer_t out,
 		return write_union(out, avro_schema_to_union(schema), parent_namespace);
 	case AVRO_LINK:
 		return write_link(out, avro_schema_to_link(schema), parent_namespace);
+	case AVRO_INVALID:
+		return EINVAL;
 	}
 
 	if (is_avro_primitive(schema)) {
